#!/usr/bin/env python3
# Build the documentation.

import errno, os, re, sys
from subprocess import check_call, CalledProcessError, Popen, PIPE, STDOUT

versions = ['1.0.0', '1.1.0', '2.0.0', '3.0.2', '4.0.0', '4.1.0', '5.0.0', '5.1.0', '5.2.0', '5.2.1', '5.3.0', '6.0.0', '6.1.0', '6.1.1', '6.1.2', '6.2.0', '6.2.1', '7.0.0', '7.0.1', '7.0.2', '7.0.3', '7.1.0', '7.1.1', '7.1.2', '7.1.3', '8.0.0', '8.0.1', '8.1.0', '8.1.1', '9.0.0', '9.1.0']

class Pip:
  def __init__(self, venv_dir):
    self.path = os.path.join(venv_dir, 'bin', 'pip')

  def install(self, package, commit=None):
    "Install package using pip."
    if commit:
      package = 'git+https://github.com/{0}.git@{1}'.format(package, commit)
    print('Installing {0}'.format(package))
    check_call([self.path, 'install', package])

def create_build_env(venv_dir='virtualenv'):
  # Create virtualenv.
  if not os.path.exists(venv_dir):
    check_call(['python3', '-m', 'venv', venv_dir])
  # Install Sphinx and Breathe. Require the exact version of Sphinx which is
  # compatible with Breathe.
  pip = Pip(venv_dir)
  pip.install('wheel')
  pip.install('six')
  # See: https://github.com/sphinx-doc/sphinx/issues/9777
  pip.install('docutils==0.17.1')
  # Jinja2 >= 3.1 incompatible with sphinx 3.3.0
  # See: https://github.com/sphinx-doc/sphinx/issues/10291
  pip.install('Jinja2<3.1')
  pip.install('sphinx-doc/sphinx', 'v3.3.0')
  pip.install('michaeljones/breathe', 'v4.25.0')

def build_docs(version='dev', **kwargs):
  doc_dir = kwargs.get('doc_dir', os.path.dirname(os.path.realpath(__file__)))
  work_dir = kwargs.get('work_dir', '.')
  include_dir = kwargs.get(
      'include_dir', os.path.join(os.path.dirname(doc_dir), 'include', 'fmt'))
  # Build docs.
  cmd = ['doxygen', '-']
  p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=STDOUT)
  doxyxml_dir = os.path.join(work_dir, 'doxyxml')
  out, _ = p.communicate(input=r'''
      PROJECT_NAME      = fmt
      GENERATE_LATEX    = NO
      GENERATE_MAN      = NO
      GENERATE_RTF      = NO
      CASE_SENSE_NAMES  = NO
      INPUT             = {0}/chrono.h {0}/color.h {0}/core.h {0}/compile.h \
                          {0}/format.h {0}/os.h {0}/ostream.h {0}/printf.h \
                          {0}/xchar.h
      QUIET             = YES
      JAVADOC_AUTOBRIEF = YES
      AUTOLINK_SUPPORT  = NO
      GENERATE_HTML     = NO
      GENERATE_XML      = YES
      XML_OUTPUT        = {1}
      ALIASES           = "rst=\verbatim embed:rst"
      ALIASES          += "endrst=\endverbatim"
      MACRO_EXPANSION   = YES
      PREDEFINED        = _WIN32=1 \
                          __linux__=1 \
                          FMT_ENABLE_IF(...)= \
                          FMT_USE_VARIADIC_TEMPLATES=1 \
                          FMT_USE_RVALUE_REFERENCES=1 \
                          FMT_USE_USER_DEFINED_LITERALS=1 \
                          FMT_USE_ALIAS_TEMPLATES=1 \
                          FMT_USE_NONTYPE_TEMPLATE_ARGS=1 \
                          FMT_API= \
                          "FMT_BEGIN_NAMESPACE=namespace fmt {{" \
                          "FMT_END_NAMESPACE=}}" \
                          "FMT_STRING_ALIAS=1" \
                          "FMT_VARIADIC(...)=" \
                          "FMT_VARIADIC_W(...)=" \
                          "FMT_DOC=1"
      EXCLUDE_SYMBOLS   = fmt::formatter fmt::printf_formatter fmt::arg_join \
                          fmt::basic_format_arg::handle
    '''.format(include_dir, doxyxml_dir).encode('UTF-8'))
  out = out.decode('utf-8')
  internal_symbols = [
    'fmt::detail::.*',
    'basic_data<>',
    'fmt::type_identity'
  ]
  noisy_warnings = [
    'warning: (Compound|Member .* of class) (' + '|'.join(internal_symbols) + \
      ') is not documented.',
    'warning: Internal inconsistency: .* does not belong to any container!'
  ]
  for w in noisy_warnings:
      out = re.sub('.*' + w + '\n', '', out)
  print(out)
  if p.returncode != 0:
    raise CalledProcessError(p.returncode, cmd)

  html_dir = os.path.join(work_dir, 'html')
  main_versions = reversed(versions[-3:])
  check_call([os.path.join(work_dir, 'virtualenv', 'bin', 'sphinx-build'),
              '-Dbreathe_projects.format=' + os.path.abspath(doxyxml_dir),
              '-Dversion=' + version, '-Drelease=' + version,
              '-Aversion=' + version, '-Aversions=' + ','.join(main_versions),
              '-b', 'html', doc_dir, html_dir])
  try:
    check_call(['lessc', '--verbose', '--clean-css',
                '--include-path=' + os.path.join(doc_dir, 'bootstrap'),
                os.path.join(doc_dir, 'fmt.less'),
                os.path.join(html_dir, '_static', 'fmt.css')])
  except OSError as e:
    if e.errno != errno.ENOENT:
      raise
    print('lessc not found; make sure that Less (http://lesscss.org/) ' +
          'is installed')
    sys.exit(1)
  return html_dir

if __name__ == '__main__':
  create_build_env()
  build_docs(sys.argv[1])



from typing import Any, Literal, Never, overload
from typing import Sequence as _Sequence, Union as _Union

import numpy as np
import numpy.typing as npt


class PathType:
    Subject: PathType = ...
    Clip: PathType = ...

Subject: PathType = ...
Clip: PathType = ...


class ClipType:
    Union: ClipType = ...
    Difference: ClipType = ...
    Intersection: ClipType = ...
    Xor: ClipType = ...

Union: ClipType = ...
Difference: ClipType = ...
Intersection: ClipType = ...
Xor: ClipType = ...


class FillRule:
    EvenOdd: FillRule = ...
    NonZero: FillRule = ...
    Positive: FillRule = ...
    Negative: FillRule = ...

EvenOdd: FillRule = ...
NonZero: FillRule = ...
Positive: FillRule = ...
Negative: FillRule = ...


class JoinType:
    Square: JoinType = ...
    Round: JoinType = ...
    Miter: JoinType = ...

# Square: JoinType = ...  # FIXME
# Round: JoinType = ...  # FIXME
Miter: JoinType = ...


class EndType:
    Square: EndType = ...
    Butt: EndType = ...
    Joined: EndType = ...
    Polygon: EndType = ...
    Round: EndType = ...

Square: EndType = ...  # FIXME
Butt: EndType = ...
Joined: EndType = ...
Polygon: EndType = ...
Round: EndType = ...  # FIXME


clipperVersion: str = ...


class PolyPath:
    def __init__(self) -> Never: ...

    @property
    def level(self) -> int: ...

    @property
    def parent(self) -> PolyPath: ...


class PolyTree:
    def __init__(self) -> Never: ...

    @property
    def isHole(self) -> bool: ...

    @property
    def area(self) -> float: ...

    @property
    def attributes(self) -> npt.NDArray[np.float64]: ...

    @property
    def polygon(self) -> npt.NDArray[np.float64]: ...

    @property
    def children(self) -> list[PolyTree]: ...

    @property
    def count(self) -> int: ...

    def __len__(self) -> int: ...


class PolyTreeD:
    def __init__(self) -> Never: ...

    @property
    def isHole(self) -> bool: ...

    @property
    def area(self) -> float: ...

    @property
    def attributes(self) -> npt.NDArray[np.float64]: ...

    @property
    def polygon(self) -> npt.NDArray[np.float64]: ...

    @property
    def children(self) -> list[PolyTreeD]: ...

    @property
    def count(self) -> int: ...

    def __len__(self) -> int: ...


def polyTreeToPaths64(arg0: PolyTree) -> Any: ...  # TODO
def orientation(path: Any, scaleFactor: float) -> bool: ...  # TODO
def polyTreeToPaths(arg0: PolyTree) -> Any: ...  # TODO
def simplifyPath(path: Any, epsilon: float, isOpenPath: bool = ...) -> Any: ...  # TODO
def simplifyPaths(paths: Any, epsilon: float, isOpenPath: bool = ...) -> Any: ...  # TODO


class Clipper:
    scaleFactor: float
    preserveCollinear: bool

    @overload
    def addPath(
        self,
        path: npt.NDArray[np.float64],
        pathType: PathType,
        isOpen: bool = ...
    ) -> None: ...

    @overload
    def addPath(
        self,
        path: _Sequence[_Sequence[_Union[int, float]]],
        pathType: PathType,
        isOpen: bool = ...
    ) -> None: ...

    @overload
    def addPaths(
        self,
        paths: list[npt.NDArray[np.float64]],
        pathType: PathType,
        isOpen: bool = ...
    ) -> None: ...

    @overload
    def addPaths(
        self,
        paths: _Sequence[_Sequence[_Sequence[_Union[int, float]]]],
        pathType: PathType,
        isOpen: bool = ...
    ) -> None: ...

    @overload
    def execute(
        self,
        clipType: ClipType,
        fillRule: FillRule = ...,
        *,
        returnOpenPaths: Literal[False] = ...,
        returnZ: Literal[False] = ...
    ) -> list[npt.NDArray[np.float64]]: ...

    @overload
    def execute(
        self,
        clipType: ClipType,
        fillRule: FillRule = ...,
        *,
        returnOpenPaths: Literal[False] = ...,
        returnZ: Literal[True] = ...
    ) -> tuple[
        list[npt.NDArray[np.float64]],
        list[npt.NDArray[np.float64]],
        list[npt.NDArray[np.float64]]
    ]: ...

    @overload
    def execute(
        self,
        clipType: ClipType,
        fillRule: FillRule = ...,
        *,
        returnOpenPaths: Literal[True] = ...,
        returnZ: bool = ...
    ) -> tuple[
        list[npt.NDArray[np.float64]],
        list[npt.NDArray[np.float64]],
        list[npt.NDArray[np.float64]],
        list[npt.NDArray[np.float64]]
    ]: ...

    @overload
    def execute2(
        self,
        clipType: ClipType,
        fillRule: FillRule = ...,
        *,
        returnOpenPaths: Literal[False] = ...,
        returnZ: bool = ...
    ) -> PolyTreeD: ...

    @overload
    def execute2(
        self,
        clipType: ClipType,
        fillRule: FillRule = ...,
        *,
        returnOpenPaths: Literal[True] = ...,
        returnZ: bool = ...
    ) -> tuple[
        PolyTreeD,
        list[npt.NDArray[np.float64]],
        list[npt.NDArray[np.float64]]
    ]: ...

    @overload
    def executeTree(
        self,
        clipType: ClipType,
        fillRule: FillRule = ...,
        *,
        returnOpenPaths: Literal[False] = ...,
        returnZ: bool = ...
    ) -> PolyTreeD: ...

    @overload
    def executeTree(
        self,
        clipType: ClipType,
        fillRule: FillRule = ...,
        *,
        returnOpenPaths: Literal[True] = ...,
        returnZ: bool = ...
    ) -> tuple[
        PolyTreeD,
        list[npt.NDArray[np.float64]],
        list[npt.NDArray[np.float64]]
    ]: ...

    def clear(self) -> None: ...

    def cleanUp(self) -> None: ...


class ClipperOffset:
    scaleFactor: float
    arcTolerance: float
    miterLimit: float
    preserveCollinear: bool

    def addPath(
        self,
        path: npt.NDArray[np.float64],
        joinType: JoinType,
        endType: EndType = ...
    ) -> None: ...

    def addPaths(
        self,
        paths: list[npt.NDArray[np.float64]],
        joinType: JoinType,
        endType: EndType = ...
    ) -> None: ...

    def execute(
        self,
        delta: float
    ) -> list[npt.NDArray[np.float64]]: ...

    def execute2(
        self,
        delta: float
    ) -> PolyTreeD: ...

    def clear(self) -> None: ...


def minkowskiSum(
    pattern: npt.NDArray[np.float64],
    path: npt.NDArray[np.float64],
    isClosed: bool,
    scaleFactor: float = ...
) -> list[npt.NDArray[np.float64]]: ...
def minkowskiDiff(
    pattern: npt.NDArray[np.float64],
    path: npt.NDArray[np.float64],
    isClosed: bool,
    scaleFactor: float = ...
) -> list[npt.NDArray[np.float64]]: ...


__version__: str = ...



import trampoline_module


def func():
    class Test(trampoline_module.test_override_cache_helper):
        def func(self):
            return 42

    return Test()


def func2():
    class Test(trampoline_module.test_override_cache_helper):
        pass

    return Test()



# Copyright (c) 2024 The pybind Community.

import exo_planet_c_api
import exo_planet_pybind11
import home_planet_very_lonely_traveler
import pytest

from pybind11_tests import cpp_conduit as home_planet


def test_traveler_getattr_actually_exists():
    t_h = home_planet.Traveler("home")
    assert t_h.any_name == "Traveler GetAttr: any_name luggage: home"


def test_premium_traveler_getattr_actually_exists():
    t_h = home_planet.PremiumTraveler("home", 7)
    assert t_h.secret_name == "PremiumTraveler GetAttr: secret_name points: 7"


def test_call_cpp_conduit_success():
    t_h = home_planet.Traveler("home")
    cap = t_h._pybind11_conduit_v1_(
        home_planet.PYBIND11_PLATFORM_ABI_ID,
        home_planet.cpp_type_info_capsule_Traveler,
        b"raw_pointer_ephemeral",
    )
    assert cap.__class__.__name__ == "PyCapsule"


def test_call_cpp_conduit_platform_abi_id_mismatch():
    t_h = home_planet.Traveler("home")
    cap = t_h._pybind11_conduit_v1_(
        home_planet.PYBIND11_PLATFORM_ABI_ID + b"MISMATCH",
        home_planet.cpp_type_info_capsule_Traveler,
        b"raw_pointer_ephemeral",
    )
    assert cap is None


def test_call_cpp_conduit_cpp_type_info_capsule_mismatch():
    t_h = home_planet.Traveler("home")
    cap = t_h._pybind11_conduit_v1_(
        home_planet.PYBIND11_PLATFORM_ABI_ID,
        home_planet.cpp_type_info_capsule_int,
        b"raw_pointer_ephemeral",
    )
    assert cap is None


def test_call_cpp_conduit_pointer_kind_invalid():
    t_h = home_planet.Traveler("home")
    with pytest.raises(
        RuntimeError, match='^Invalid pointer_kind: "raw_pointer_ephemreal"$'
    ):
        t_h._pybind11_conduit_v1_(
            home_planet.PYBIND11_PLATFORM_ABI_ID,
            home_planet.cpp_type_info_capsule_Traveler,
            b"raw_pointer_ephemreal",
        )


def test_home_only_basic():
    t_h = home_planet.Traveler("home")
    assert t_h.luggage == "home"
    assert home_planet.get_luggage(t_h) == "home"


def test_home_only_premium():
    p_h = home_planet.PremiumTraveler("home", 2)
    assert p_h.luggage == "home"
    assert home_planet.get_luggage(p_h) == "home"
    assert home_planet.get_points(p_h) == 2


def test_exo_only_basic():
    t_e = exo_planet_pybind11.Traveler("exo")
    assert t_e.luggage == "exo"
    assert exo_planet_pybind11.get_luggage(t_e) == "exo"


def test_exo_only_premium():
    p_e = exo_planet_pybind11.PremiumTraveler("exo", 3)
    assert p_e.luggage == "exo"
    assert exo_planet_pybind11.get_luggage(p_e) == "exo"
    assert exo_planet_pybind11.get_points(p_e) == 3


def test_home_passed_to_exo_basic():
    t_h = home_planet.Traveler("home")
    assert exo_planet_pybind11.get_luggage(t_h) == "home"


def test_exo_passed_to_home_basic():
    t_e = exo_planet_pybind11.Traveler("exo")
    assert home_planet.get_luggage(t_e) == "exo"


def test_home_passed_to_exo_premium():
    p_h = home_planet.PremiumTraveler("home", 2)
    assert exo_planet_pybind11.get_luggage(p_h) == "home"
    assert exo_planet_pybind11.get_points(p_h) == 2


def test_exo_passed_to_home_premium():
    p_e = exo_planet_pybind11.PremiumTraveler("exo", 3)
    assert home_planet.get_luggage(p_e) == "exo"
    assert home_planet.get_points(p_e) == 3


@pytest.mark.parametrize(
    "traveler_type", [home_planet.Traveler, exo_planet_pybind11.Traveler]
)
def test_exo_planet_c_api_traveler(traveler_type):
    t = traveler_type("socks")
    assert exo_planet_c_api.GetLuggage(t) == "socks"


@pytest.mark.parametrize(
    "premium_traveler_type",
    [home_planet.PremiumTraveler, exo_planet_pybind11.PremiumTraveler],
)
def test_exo_planet_c_api_premium_traveler(premium_traveler_type):
    pt = premium_traveler_type("gucci", 5)
    assert exo_planet_c_api.GetLuggage(pt) == "gucci"
    assert exo_planet_c_api.GetPoints(pt) == 5


def test_home_planet_wrap_very_lonely_traveler():
    # This does not exercise the cpp_conduit feature, but is here to
    # demonstrate that the cpp_conduit feature does not solve all
    # cross-extension interoperability issues.
    # Here is the proof that the following works for extensions with
    # matching `PYBIND11_INTERNALS_ID`s:
    #     test_cpp_conduit.cpp:
    #         py::class_<LonelyTraveler>
    #     home_planet_very_lonely_traveler.cpp:
    #         py::class_<VeryLonelyTraveler, LonelyTraveler>
    # See test_exo_planet_pybind11_wrap_very_lonely_traveler() for the negative
    # test.
    assert home_planet.LonelyTraveler is not None  # Verify that the base class exists.
    home_planet_very_lonely_traveler.wrap_very_lonely_traveler()
    # Ensure that the derived class exists.
    assert home_planet_very_lonely_traveler.VeryLonelyTraveler is not None


def test_exo_planet_pybind11_wrap_very_lonely_traveler():
    # See comment under test_home_planet_wrap_very_lonely_traveler() first.
    # Here the `PYBIND11_INTERNALS_ID`s don't match between:
    #     test_cpp_conduit.cpp:
    #         py::class_<LonelyTraveler>
    #     exo_planet_pybind11.cpp:
    #         py::class_<VeryLonelyTraveler, LonelyTraveler>
    assert home_planet.LonelyTraveler is not None  # Verify that the base class exists.
    with pytest.raises(
        RuntimeError,
        match='^generic_type: type "VeryLonelyTraveler" referenced unknown base type '
        '"pybind11_tests::test_cpp_conduit::LonelyTraveler"$',
    ):
        exo_planet_pybind11.wrap_very_lonely_traveler()



import os
import re
import sys
import platform
import subprocess

from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext

from setuptools.command.develop import develop
from setuptools.command.install import install
from setuptools.command.sdist import sdist
from setuptools.command.build_py import build_py
from setuptools import setup, find_packages
from subprocess import check_call, check_output

with open('README.rst') as f:
    readme = f.read()

""" Specific versions of submodules to use. """

submoduleVersions = {
    'clipper2': 'Clipper2_1.3.0',
    'pybind11': 'v2.11',
    'eigen': '3.4'
}

# Remote URLs for submodules to ensure fetch works
submoduleRemotes = {
    'clipper2': 'https://github.com/AngusJohnson/Clipper2.git',
    'pybind11': 'https://github.com/pybind/pybind11.git',
    'eigen': 'https://gitlab.com/libeigen/eigen.git'
}

class CMakeExtension(Extension):
    def __init__(self, name, modName, sourcedir=''):

        Extension.__init__(self, name, sources=[], language='c++')

        self.modName = modName
        self.sourcedir = os.path.abspath(sourcedir)


def gitcmd_update_submodules():
    """
    Check if the package is being deployed as a git repository. If so, recursively update all dependencies.
    """

    if os.path.exists('.git'):

        check_call(['git', 'submodule', 'update', '--init', '--recursive'])

        # set all versions as required
        curPath = os.path.abspath('.')

        for k, v in submoduleVersions.items():
            modPath =  os.path.abspath('{:s}/external/{:s}'.format(curPath, k))

            import subprocess

            # Ensure origin remote is set
            remote_url = submoduleRemotes[k]
            try:
                check_output(['git', 'remote', 'get-url', 'origin'], cwd=modPath)
            except subprocess.CalledProcessError:
                check_call(['git', 'remote', 'add', 'origin', remote_url], cwd=modPath)

            # Fetch only the specific required tag to avoid conflicts
            check_call(['git', 'fetch', 'origin', f'{v}:refs/tags/{v}'], cwd=modPath)
            # Use check_call to raise on failure
            check_call(['git', 'checkout', v], cwd=modPath)


        check_call(['git', 'submodule', 'status'])

        return True

    return False


class gitcmd_develop(develop):
    """
    Specialized packaging class that runs `git submodule update --init --recursive`
    as part of the update/install procedure.
    """
    def run(self):
        gitcmd_update_submodules()
        develop.run(self)


class gitcmd_install(install):
    """
    Specialized packaging class that runs `git submodule update --init --recursive`
    as part of the update/install procedure.
    """
    def run(self):
        gitcmd_update_submodules()
        install.run(self)

class gitcmd_sdist(sdist):
    """
    Specialized packaging class that runs git submodule update --init --recursive
    as part of the update/install procedure;.
    """
    def run(self):
        gitcmd_update_submodules()
        sdist.run(self)

class CMakeBuild(build_ext):
    def run(self):

        # Update the submodules to the correct version
        gitcmd_update_submodules()

        try:
            out = subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError("CMake must be installed to build the following extensions: " +
                               ", ".join(e.name for e in self.extensions))

        if platform.system() == "Windows":
            match = re.search(r'version\s*([\d.]+)', out.decode())
            if not match:
                raise RuntimeError("Could not parse CMake version from output")
            parts = [int(p) for p in match.group(1).split('.')]
            if len(parts) < 2 or parts[0] < 3 or (parts[0] == 3 and parts[1] < 12):
                raise RuntimeError("CMake >= 3.12.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        """
        Builds the following CMake extension within the project tree

        :param ext: CMake Extension to build
        """
        print('ext_full_path', self.get_ext_fullpath(ext.name))
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        # required for auto-detection of auxiliary "native" libs
        if not extdir.endswith(os.path.sep):
            extdir += os.path.sep

        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir,
                      '-DPYTHON_EXECUTABLE=' + sys.executable,
                      '-DBUILD_PYTHON=ON']

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(cfg.upper(), extdir)]
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]
            build_args += ['--', '-j4']

        cmake_args += ['-DCMAKE_INSTALL_PREFIX=' + '.']

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(env.get('CXXFLAGS', ''),
                                                              self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args, cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.', '--target', ext.modName] + build_args, cwd=self.build_temp)


setup(
    name='pyclipr2',
    version='0.1.9',
    author='2ico',
    author_email='duico@pm.me',
    url='https://github.com/2ico/pyclipr',
    long_description=readme,
    long_description_content_type = 'text/plain',
    description='Python library for polygon clipping and offsetting based on Clipper2.',
    ext_modules=[CMakeExtension('pyclipr.pyclipr', 'pyclipr')],
    cmdclass= {
        'build_ext': CMakeBuild,
        'develop': gitcmd_develop,
        'sdist': gitcmd_sdist,
    },
    packages = ['pyclipr'],
    package_dir={'': 'python'},
    keywords=['polygon clipping', 'polygon offsetting', 'libClipper', 'Clipper2', 'polygon boolean', 'polygon', 'line clipping', 'clipper'],
    # tell setuptools that all packages will be under the 'src' directory
    # and nowhere else
    zip_safe=False,
        classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Natural Language :: English',
        'Topic :: Scientific/Engineering'],
    license="",
    project_urls = {
        'Documentation': 'https://github.com/drylukeparry/pyclipr',
        'Source': 'https://github.com/drylukeparry/pyclipr',
        'Tracker': 'https://github.com/drlukeparry/pyclipr/issues'
    }
)




#!/usr/bin/env python
#
# Copyright 2009, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Tests that leaked mock objects can be caught be Google Mock."""

from googlemock.test import gmock_test_utils

PROGRAM_PATH = gmock_test_utils.GetTestExecutablePath('gmock_leak_test_')
TEST_WITH_EXPECT_CALL = [PROGRAM_PATH, '--gtest_filter=*ExpectCall*']
TEST_WITH_ON_CALL = [PROGRAM_PATH, '--gtest_filter=*OnCall*']
TEST_MULTIPLE_LEAKS = [PROGRAM_PATH, '--gtest_filter=*MultipleLeaked*']

environ = gmock_test_utils.environ
SetEnvVar = gmock_test_utils.SetEnvVar

# Tests in this file run a Google-Test-based test program and expect it
# to terminate prematurely.  Therefore they are incompatible with
# the premature-exit-file protocol by design.  Unset the
# premature-exit filepath to prevent Google Test from creating
# the file.
SetEnvVar(gmock_test_utils.PREMATURE_EXIT_FILE_ENV_VAR, None)


class GMockLeakTest(gmock_test_utils.TestCase):

  def testCatchesLeakedMockByDefault(self):
    self.assertNotEqual(
        0,
        gmock_test_utils.Subprocess(TEST_WITH_EXPECT_CALL,
                                    env=environ).exit_code)
    self.assertNotEqual(
        0,
        gmock_test_utils.Subprocess(TEST_WITH_ON_CALL,
                                    env=environ).exit_code)

  def testDoesNotCatchLeakedMockWhenDisabled(self):
    self.assertEquals(
        0,
        gmock_test_utils.Subprocess(TEST_WITH_EXPECT_CALL +
                                    ['--gmock_catch_leaked_mocks=0'],
                                    env=environ).exit_code)
    self.assertEquals(
        0,
        gmock_test_utils.Subprocess(TEST_WITH_ON_CALL +
                                    ['--gmock_catch_leaked_mocks=0'],
                                    env=environ).exit_code)

  def testCatchesLeakedMockWhenEnabled(self):
    self.assertNotEqual(
        0,
        gmock_test_utils.Subprocess(TEST_WITH_EXPECT_CALL +
                                    ['--gmock_catch_leaked_mocks'],
                                    env=environ).exit_code)
    self.assertNotEqual(
        0,
        gmock_test_utils.Subprocess(TEST_WITH_ON_CALL +
                                    ['--gmock_catch_leaked_mocks'],
                                    env=environ).exit_code)

  def testCatchesLeakedMockWhenEnabledWithExplictFlagValue(self):
    self.assertNotEqual(
        0,
        gmock_test_utils.Subprocess(TEST_WITH_EXPECT_CALL +
                                    ['--gmock_catch_leaked_mocks=1'],
                                    env=environ).exit_code)

  def testCatchesMultipleLeakedMocks(self):
    self.assertNotEqual(
        0,
        gmock_test_utils.Subprocess(TEST_MULTIPLE_LEAKS +
                                    ['--gmock_catch_leaked_mocks'],
                                    env=environ).exit_code)


if __name__ == '__main__':
  gmock_test_utils.Main()
